import DataRow
import LoadCsvData


class LoadFeatureData:
    """ A class for adding calculated features to the data rows loaded from csv. """

    rows = []
    rowCount = 0

    limitLoad = True
    rowLimit = 25

    cleanData = False
    cleanCount = 0

    verbose = False
    loadCsvData = None

    def __init__(self, lLimitLoad=False, lRowLimit=-1, lCleanData=False, lVerbose=False, lLoadCsvData=None):
        self.limitLoad = lLimitLoad
        self.rowLimit = lRowLimit
        self.cleanData = lCleanData
        self.verbose = lVerbose
        self.loadCsvData = lLoadCsvData
    #edef

    def generateData(self, type='goog_stock_sma100'):
        print ("")
        print ("")
        print("Generating Feature Data: Type: " + type)

        if type == '':
            self.rowCount = self.loadCsvData.rowCount
            rownum = self.rowCount

            lrows = []
            lrows.extend(self.loadCsvData.rows)

            print ("Loaded %i rows from this data file." % (rownum))
            lrows = self.sortRows(lrows)
            self.cleanRows(lrows)
            self.rows.extend(lrows)
            print ('CleanCount: %i RowCount: %i RowsFound: %i' % (self.cleanCount, self.rowCount, len(self.rows)))

        elif type == 'weight_age':
            self.rowCount = self.loadCsvData.rowCount
            rownum = self.rowCount

            lrows = []
            lrows.extend(self.loadCsvData.rows)

            for i in xrange(len(lrows)):
                # The answer in linear regression models will always be stored in the 'Answer' column
                lrows[i].setMember('Answer', 3, float(lrows[i].getMemberByName('BloodFat')))
            # efl

            print ("Loaded %i rows from this data file." % (rownum))
            lrows = self.sortRows(lrows)
            self.cleanRows(lrows)
            self.rows.extend(lrows)
            print ('CleanCount: %i RowCount: %i RowsFound: %i' % (self.cleanCount, self.rowCount, len(self.rows)))

        elif type == 'goog_stock_sma100':
            self.resetRows()

            lrows = []
            lrows.extend(self.loadCsvData.rows)
            rownum = 0
            avg = 0
            target = 100
            cnt = 0
            startingVal = lrows[0].getMemberByName('Close')

            for i in xrange(len(lrows)):
                try:
                    float(lrows[i].getMemberByName('Close'))
                    float(lrows[i].getMemberByName('Open'))
                    float(lrows[i].getMemberByName('High'))
                    float(lrows[i].getMemberByName('Low'))
                except:
                    lrows[i].error = True
            # efl
            self.cleanRows(lrows)

            for i in xrange(len(lrows)):
                # The answer in linear regression models will always be stored in the 'Answer' column
                lrows[i].setMember('Answer', 8, float(lrows[i].getMemberByName('Close')))

                if i < target:
                    lrows[i].setMember('sma_100', 7, startingVal)
                else:
                    avg = 0

                    for j in xrange(len(lrows)):
                        if j < (i - target):
                            ''' do nothing '''
                        else:
                            if j >= (i - target) and j <= (i - 0):
                                avg += float(lrows[j].getMemberByName('Close'))
                            elif j > (i - 0):
                                break
                            # eif
                        # eif
                    # efl

                    if self.verbose == True:
                        print('sma_100: %d' % (float(avg) / float(target)))
                    # eif

                    lrows[i].setMember('sma_100', 7, (float(avg) / float(target)))
                # eif

                if self.limitLoad == True and cnt >= self.rowLimit and self.rowLimit > 0:
                    break;
                # eif

                rownum += 1
                cnt += 1
            # efl

            print ("Loaded %i rows from this data file." % (rownum))
            lrows = self.sortRows(lrows)
            self.cleanRows(lrows)
            self.rows.extend(lrows)
            self.rowCount = len(lrows)

            print ('CleanCount: %i RowCount: %i RowsFound: %i' % (self.cleanCount, self.rowCount, len(self.rows)))

        # eif
    # edef

    def resetRows(self):
        self.rows = []
        self.rowCount = 0
        self.cleanCount = 0
    # edef

    def sortRows(self, lrows):
        return sorted(lrows, key=id)
    # edef

    def cleanRows(self, lrows):
        if self.cleanData == True:
            print ("Cleaning row data...")
            should_restart = True
            while should_restart:
                should_restart = False
                for row in lrows:
                    if row.error == True:
                        lrows.remove(row)
                        self.rowCount -= 1
                        self.cleanCount += 1
                        should_restart = True
                    # eif
                # efl
            # fwl
        # eif
    # edef

# eclass
