#!/usr/bin/env python

# base imports
import sys
import tensorflow as tf

# user imports
import LoadCsvData
import Data2DataRow
import LoadFeatureData
import LoadTensorData
import DataRow2Tensor
import RegModelLinear


dataDir = "./data"
appVersion = "0.4.0.7_python2.7"

def run(exeCfg):
    type = exeCfg['type']
    data_2_datarow_type = exeCfg['data_2_datarow_type']
    datarow_2_tensor_type = exeCfg['datarow_2_tensor_type']
    version = exeCfg['version']
    reset = exeCfg['reset']
    checkpoint = exeCfg['checkpoint']
    verbose = exeCfg['verbose']
    limitLoad = exeCfg['limitLoad']
    rowLimit = exeCfg['rowLimit']
    validatePrct = exeCfg['validatePrct']
    trainPrct = exeCfg['trainPrct']
    randomSeed = exeCfg['randomSeed']
    learning_rate = exeCfg['learning_rate']
    log_reg_positive_result = exeCfg['log_reg_positive_result']
    lin_reg_positive_result = exeCfg['lin_reg_positive_result']
    model_type = exeCfg['model_type']
    loader = exeCfg['loader']
    cleanData = exeCfg['cleanData']
    trainStepsMultiplier = exeCfg['trainStepsMultiplier']
    dataMap = Data2DataRow.mapping[data_2_datarow_type]
    files = exeCfg['files']
    featureType = exeCfg['feature_type']
    logPrint = exeCfg['logPrint']
    evalType = exeCfg['eval_type']
    data = None
    fData = None
    tData = None
    tfModel = None

    print("Found loader: " + loader)
    if loader == 'load_csv_data':
        data = LoadCsvData.LoadCsvData()
        data.limitLoad = limitLoad
        data.rowLimit = rowLimit
        data.verbose = verbose

        for file in files:
            csvFileName = files[file]['name']
            appendCols = files[file]['appendCols']
            data.loadData(csvFileName, type, version, reset, dataMap, appendCols)
        # efl
    # eif

    if featureType != '':
        print("Found feature type: " + featureType)
        fData = LoadFeatureData.LoadFeatureData(limitLoad, rowLimit, cleanData, verbose, data)
        fData.generateData(featureType)
    else:
        print("Found no feature type.")
        fData = LoadFeatureData.LoadFeatureData(limitLoad, rowLimit, cleanData, verbose, data)
        fData.generateData('')
    # eif

    tData = LoadTensorData.LoadTensorData(verbose, trainPrct, validatePrct, fData)
    tData.generateData(DataRow2Tensor.columns[datarow_2_tensor_type])

    if model_type == 'linear_regression':
        tfModel = RegModelLinear.RegModelLinear(verbose, tData, lin_reg_positive_result, randomSeed, trainStepsMultiplier, logPrint, learning_rate, evalType)
        tfModel.startTraining()
    # eif

# edef

def main(args):

    print("Application Version: " + appVersion)

    exes = {
                "goog_lin_reg_avg100day":
                {
                    'type': 'csv',
                    'data_2_datarow_type': 'google_price',
                    'datarow_2_tensor_type': 'goog_lin_reg_avg100day',
                    'version': '1.0',
                    'reset': False,
                    'checkpoint': False,
                    'limitLoad': False,
                    'cleanData': True,
                    'verbose': False,
                    'rowLimit': 125,
                    'validatePrct': 0.30,
                    'trainPrct': 0.70,
                    'randomSeed': False,
                    'trainStepsMultiplier': 1,
                    'learning_rate': 0.00000001,
                    'log_reg_positive_result': 0.00,
                    'lin_reg_positive_result': 0.50,
                    'model_type': 'linear_regression',
                    'loader': 'load_csv_data',
                    'feature_type': 'goog_stock_sma100',
                    'logPrint': 50,
                    'eval_type': 'goog_stock_sma100',

                    'files': {
                        'file1': {'name': dataDir + "/spy.csv.xls", 'appendCols': [{'Symbol': 'spy', 'idx': '6'}]},
                    }
                },

                "weight_age_lin_reg":
                {
                    'type': 'csv',
                    'data_2_datarow_type': 'weight_age',
                    'datarow_2_tensor_type': 'weight_age_lin_reg_blood_fat',
                    'version': '1.0',
                    'reset': False,
                    'checkpoint': False,
                    'limitLoad': False,
                    'cleanData': True,
                    'verbose': True,
                    'rowLimit': 25,
                    'validatePrct': 0.30,
                    'trainPrct': 1.00,
                    'randomSeed': False,
                    'trainStepsMultiplier': 100,
                    'learning_rate': 0.0001,
                    'log_reg_positive_result': 0.00,
                    'lin_reg_positive_result': 0.50,
                    'model_type': 'linear_regression',
                    'loader': 'load_csv_data',
                    'feature_type': 'weight_age',
                    'logPrint': 500,
                    'eval_type': 'weight_age',

                    'files': {
                        'file1': {'name': dataDir + "/weight_age.csv.xls", 'appendCols': []}
                    }
                }
    }

    # run(exes["goog_lin_reg_avg100day"]);
    run(exes["weight_age_lin_reg"]);

# edef

if __name__ == '__main__':
    main(sys.argv)
# eif
